﻿using System;
using AIOUSBNet;

namespace DIOSample
{
    class InputOutputClass
    {
        public static UInt32 DeviceIndex = AIOUSB.diOnly;

        public static bool WriteBit(int bitIndex, bool setBitHigh)
        {
            var result = AIOUSB.DIO_Write1(DeviceIndex, (uint)bitIndex, (setBitHigh ? (byte)0x01 : (byte)0x00));
            Console.WriteLine($"Wrote    {(setBitHigh?1:0)}    to bit at bitIndex {bitIndex}");
            if (result != 0)
            {
                Console.WriteLine($"error {result} from AIOUSB.DIO_Write1()");
                return false;
            }
            return true;
        }

        public static bool WriteByte(byte data) // this board only has one "byte" (4 bits) of output data so we skip the byteIndex parameter
        {
            var result = AIOUSB.DIO_Write8(DeviceIndex, 0, data);   
            Console.WriteLine($"Wrote    {data:X1}    to output bits");
            if (result != 0)
            {
                Console.WriteLine($"error {result} from AIOUSB.DIO_Write8()");
                return false;
            }
            return true;
        }

        public static bool ReadbackByte(out byte data) // this board only has one "byte" (4 bits) of input data so we skip the byteIndex parameter
        {
            data = 0;
            var result = AIOUSB.DIO_Read8(DeviceIndex, 0, ref data);
            Console.WriteLine($"Readback {data:X1}    from output bits");
            if (result != 0)
            {
                Console.WriteLine($"error {result} from AIOUSB.DIO_Read8()");
                return false;
            }
            return true;
        }

        public static bool ReadByte(out byte data) // this board only has one "byte" (4 bits) of input data so we skip the byteIndex parameter
        {
            data = 0;
            var result = AIOUSB.DIO_Read8(DeviceIndex, 1, ref data);
            Console.WriteLine($"Read {data:X1}    from input bits");
            if (result != 0)
            {
                Console.WriteLine($"error {result} from AIOUSB.DIO_Read8()");
                return false;
            }
            return true;
        }

        public static int Main()
        {
            if (AIOUSB.QueryDeviceInformation(DeviceIndex, out uint PID, out int NameSize, out string DeviceName, out uint DIOBytes, out uint Counters) != 0)
            {
                return 1;
            }

            Console.WriteLine(DeviceName);

            uint x = 0xFFFF;
            AIOUSB.DIO_WriteAll(DeviceIndex, ref x); // trying to fix the expected problem with DIO_Write1(); normally we'd use DIO_Configure, but this board "doesn't need it"

            int[] bitIndices = { 0, 1, 2, 3 };
            foreach (var bitIndex in bitIndices)
            {
                WriteBit(bitIndex, false);
                ReadbackByte(out byte data);
                WriteBit(bitIndex, true);
                ReadbackByte(out data);
                Console.WriteLine("");
            }

            byte[] bytes = { 0, 0xA, 0x5, 0xF, 0 };
            foreach (var outData in bytes)
            {
                WriteByte(outData);
                ReadbackByte(out byte data);
            }

            Console.WriteLine("Press any key to exit");
            Console.ReadKey();

            return 0;

        }

    }

}